"""
Module with some tools to manipulate RTKube plugin.
"""
from __future__ import absolute_import

from builtins import str
from builtins import object
import sys
import os
from threading import Thread
import datetime
import time

from PyQt5.QtCore import Qt, pyqtSignal, QObject, QSize, QSettings
from PyQt5.QtGui import (QPixmap,
                         QPainter,
                         QIcon,
                         QBrush,
                         QColor)
from PyQt5.QtWidgets import (QFileDialog,
                             QSizePolicy)

from . import settings
from .logger import CtrlCraftLogger
from .project import CtrlCraftProject

REPAINT_SEC = 1.0


class RTKubeAsyncRepaint(QObject, Thread):

    repaint = pyqtSignal()

    def __init__(self, timer):
        QObject.__init__(self)
        Thread.__init__(self)
        self.timer = timer
        self.stop = False

    def run(self):
        while not self.stop:
            time.sleep(self.timer)
            self.repaint.emit()


class RTKubeProxy(object):

    def __init__(self, iface, data, rad_widget, mobile=True, layer=None,
                 topoLayer=None):
        super().__init__()
        self.iface = iface
        self.data = data
        self.mobile = mobile
        self.layer = layer
        self.topoLayer = topoLayer
        self.rad_widget = rad_widget
        self.ready = False
        self.save = True
        self.canRefresh = False

        self.icon_path = 'rtkube_csv.png'
        self.icon_path_csv = 'rtkube_csv.png'

        plugin_dir = settings.RTKUBE_PLUGIN
        if not os.path.isabs(plugin_dir):
            dir_path = os.path.dirname(os.path.realpath(__file__))
            plugin_dir = os.path.join(dir_path, plugin_dir)
        self.pythonpath = os.path.join(plugin_dir, os.pardir)

    def init(self):
        sys.path.append(self.pythonpath)

        global RTKubeDockWidget
        try:
            from RTKube.rtkube_dockwidget import RTKubeDockWidget
            self.ready = True
        except ImportError as e:
            self.error(self.tr('Invalid RTKube plugin directory'))
            self.ready = False
            return False

        # init widget
        self.widget = RTKubeDockWidget()
        self.widget.dataToSMU.connect(self.received)

        if self.mobile:
            iconSize = int(QSettings().value('IconSize'))

            btn = self.widget.connectButton
            btn.setIconSize(QSize(iconSize, iconSize))
            btn.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)

            tab = self.widget.tabWidget
            tab.setIconSize(QSize(iconSize, iconSize))
            tab.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)

            styleSheet = ('QTabBar::tab {{ height: {}px; width: {}px; }}'
                          .format(iconSize, iconSize*2))
            CtrlCraftLogger().log(styleSheet)
            self.widget.tabWidget.setStyleSheet(styleSheet)

        self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.widget)

        self.iface.mapCanvas().mapCanvasRefreshed.connect(self.setCanRefresh)
        # init map repainter
        self.repainter = RTKubeAsyncRepaint(REPAINT_SEC)
        self.repainter.repaint.connect(self.repaint)
        self.repainter.start()

        return True

    def setCanRefresh(self):
        self.canRefresh = True

    def export(self):
        csv, __ = QFileDialog.getSaveFileName(None, 'RTKube data', '',
                                                  'CSV (*.csv);;All Files (*)')
        if csv:
            if not csv.endswith('.csv'):
                csv = '{}.{}'.format(csv, 'csv')
            self.data.probe_export(csv)

    def repaint(self):
        if self.layer and (self.widget.ser.ser.is_open or 'SIMU' in os.environ) and self.canRefresh:
            self.layer.triggerRepaint()
            if self.topoLayer:
                self.topoLayer.triggerRepaint()
            self.iface.mapCanvas().refresh()
            self.canRefresh = False

    def received(self, date, lat, lon, alt, radiometry, inclin, gammaup, gammadown, teneurup, teneurdown):
        if self.data.is_valid() and self.save:
            self.data.probe_new_data(date, lat, lon, alt, radiometry, gammaup,
                                     gammadown, teneurup, teneurdown)

            radiometry = self.data.slab_radiometry(lat, lon)
            if radiometry:
                self.rad_widget.mRadiometry.setText(str(radiometry[0]))
                self.rad_widget.mProbeRadiometry.setText(str(round(radiometry[1],2)))

    def error(self, msg):
        self.iface.messageBar().pushWarning(self.tr('Warning'), msg)
