# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartEstimatedBlocs
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from builtins import next
import os
import csv
import configparser

from PyQt5 import uic
from PyQt5.QtWidgets import (QFileDialog,
                         qApp,
                         QLabel,
                         QDialog)

from qgis.gui import QgsMessageBar
from qgis.core import Qgis

from CtrlCraft import CtrlCraftLogger

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'import.ui'))


class CtrlCraftOfficeSlabsImport(QDialog, FORM_CLASS):
    """
    Dialog to import Slabs in database.
    """

    def __init__(self, data, projIni, deposit, parent=None):
        super(CtrlCraftOfficeSlabsImport, self).__init__(parent)
        self.setupUi(self)
        self.data = data
        self.projIni = projIni
        self.deposit = deposit
        self.config = configparser.ConfigParser()
        self.config.read(self.projIni)
        self.updated = False

        self.__populateProjComboBox()
        self.mCancelButton.clicked.connect(self.__close)
        self.mImportButton.clicked.connect(self.__import)
        self.mSelectButton.clicked.connect(self.__select)
        self.mLineEdit.textChanged.connect(self.__projDef)

        self.mProgressBar.setValue(0)
        self.mLabel.setText('')

    def __close(self):
        self.close()

    def __select(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)
        dlg.setNameFilter("CSV files (*.txt *.csv)")

        filenames = []
        if dlg.exec_():
            filenames = dlg.selectedFiles()

        if filenames:
            filename = filenames[0]

            # read header to fill combobox
            with open(filename, 'r') as csvfile:
                reader = csv.reader(csvfile, delimiter=',')
                header = next(reader)

                if self.__is_valid(header):
                    self.mLineEdit.setText(filename)
                else:
                    CtrlCraftLogger().logBar('Invalid format', Qgis.Critical)

    def __populateProjComboBox(self):
        for deposit in self.config:
            self.mProjComboBox.addItem(deposit)
        self.mProjComboBox.setCurrentIndex(self.mProjComboBox.findText(self.deposit) if self.mProjComboBox.findText(self.deposit) != -1 else 0)
        self.mProjComboBox.setEnabled(False)
        return

    def __projDef(self):
        filename = self.mLineEdit.text()
        deposit = self.__getDepositTranslationName(filename)
        if self.mProjComboBox.findText(deposit) != -1 :
            self.mProjComboBox.setCurrentIndex(self.mProjComboBox.findText(deposit))

    def __getDepositTranslationName(self,filename):
        with open(filename) as import_file :
            smu_deposit = import_file.readlines()[1].split(',')[1]
        deposit = smu_deposit if smu_deposit in list(self.config.keys()) else 'DEFAULT'
        return deposit

    def __import(self):
        filename = self.mLineEdit.text()
        if not filename:
            return
        deposit = self.__getDepositTranslationName(filename)
        trans_x = float(self.config[deposit]['x']) if deposit != 'DEFAULT' else 0
        trans_y = float(self.config[deposit]['y']) if deposit != 'DEFAULT' else 0
        trans_z = float(self.config[deposit]['z']) if deposit != 'DEFAULT' else 0
        self.mSelectButton.setEnabled(False)
        self.mCancelButton.setEnabled(False)
        self.mImportButton.setEnabled(False)

        self.mLabel.setText(self.tr('Insert slabs in database...'))
        self.mProgressBar.setValue(0)
        qApp.processEvents()

        nlines = 0
        with open(filename, 'r') as csvfile:
            nlines = len(list(csv.reader(csvfile, delimiter=','))) - 1

        new_slabs = 0
        with open(filename, 'r') as csvfile:
            reader = csv.reader(csvfile, delimiter=',')

            nline = 0
            header = None
            for row in reader:
                if not header:
                    header = True
                    continue

                slabid = row[0]
                deposit = row[1]
                unit = row[2]
                panel = row[3]
                blast = row[4]
                quality = float(row[8])
                lithology = row[9]
                percentage = row[10]
                x = float(row[5]) + trans_x
                y = float(row[6]) + trans_y
                z = float(row[7]) + trans_z

                if not self.data.smu(slabid, deposit, panel, blast):
                    self.data.slab_create(slabid, deposit, panel, blast,
                                          quality, lithology, percentage,
                                          unit, x, y, z)
                    new_slabs += 1

                # Update progress bar
                nline += 1
                total = nline*100/nlines
                self.mProgressBar.setValue(total)

            self.mProgressBar.setValue(100)

            self.mLabel.setText(self.tr('Create index...'))
            qApp.processEvents()
            self.mProgressBar.setValue(0)
            self.mProgressBar.setMinimum(0)
            self.mProgressBar.setMaximum(0)
            self.data.slabs_create_index()

            self.mLabel.setText(self.tr('Statistics calculation...'))
            qApp.processEvents()
            self.data.estimated_bloc_stats()

            msg = (self.tr('{} slabs imported with success ({} ignored)!')
                   .format(new_slabs, nlines-new_slabs))
            self.mLabel.setText(msg)

            self.mProgressBar.setMaximum(100)
            self.mProgressBar.setValue(100)

            self.updated = True

        self.mCancelButton.setEnabled(True)

    def __is_valid(self, header):
        valid_header = [
            'Id',
            'Deposit',
            'Unit',
            'Panel',
            'Blast',
            'X',
            'Y',
            'Z',
            'U_KRIGEE',
            'LITHOLOGY',
            'LITHO_%']

        return header == valid_header
